grammar UQM2ConvoScript;

/*
 * Grammar for parsing UQM2 conversation scripts
 */

// Parser rules:
convo: choices decisions statements states items EOF;

choices: CHOICE_HEADER choice += segment+;
decisions: DECISION_HEADER decision += segment+;
statements: STATEMENT_HEADER statement += segment+;
states: STATE_HEADER state += segment*;
items: ITEM_HEADER item += segment*;

header:
	HEADER_START identifier = (
		UPPERCASE_WORD
		| STATEMENT_IDENTIFIER
	) NEWLINE;

i_remove:
	KW_REMOVE WHITESPACE+ decision = UPPERCASE_WORD WHITESPACE+ choice = UPPERCASE_WORD;

i_add:
	KW_ADD WHITESPACE+ decision = UPPERCASE_WORD WHITESPACE+ choice = UPPERCASE_WORD;

i_display:
	KW_DISPLAY WHITESPACE+ statement = STATEMENT_IDENTIFIER;

i_inc: KW_INC WHITESPACE+ variable = UPPERCASE_WORD;
i_dec: KW_DEC WHITESPACE+ variable = UPPERCASE_WORD;

i_conditional:
	KW_IF WHITESPACE+ variable = UPPERCASE_WORD WHITESPACE+ comparison = comparator WHITESPACE+
		value = (NUMBER | LOWERCASE_WORD) WHITESPACE* NEWLINE body = block NEWLINE? KW_ENDIF;

// i_replace: TODO
i_goto: KW_GOTO WHITESPACE+ decision = UPPERCASE_WORD;

comparator: C_LT | C_GT | C_LTE | C_GTE | C_EQ;

text: line+;

line: (
		WORD
		| LOWERCASE_WORD
		| UPPERCASE_WORD
		| PUNCTUATION
		| WHITESPACE
	)+ NEWLINE;

segment: header block;

block:
	statement = STATEMENT_IDENTIFIER? NEWLINE? (
		i_remove
		| i_add
		| i_display
		| i_goto
		| i_conditional
		| i_inc
		| i_dec
		| NEWLINE
	)* text?;

// Lexer rules:
fragment LOWERCASE: [a-z];
fragment UPPERCASE: [A-Z];
fragment NUMBER: [0-9];

HEADER_START: '#';

WHITESPACE: (' ' | '\t');

KW_REMOVE: 'REMOVE';
KW_ADD: 'ADD';
KW_DISPLAY: 'DISPLAY';
KW_GOTO: 'GOTO';
KW_INC: 'INC';
KW_DEC: 'DEC';
KW_IF: 'IF';
KW_ENDIF: 'ENDIF';

NEWLINE: ('\r'? '\n' | '\r')+;

CHOICE_HEADER: '@PLAYER_CHOICES@' NEWLINE;
DECISION_HEADER: '@DECISIONS@' NEWLINE;
STATEMENT_HEADER: '@STATEMENTS@' NEWLINE;
STATE_HEADER: '@STATES@' NEWLINE;
ITEM_HEADER: '@ITEMS@' NEWLINE;

STATEMENT_IDENTIFIER:
	'STATEMENT_' (UPPERCASE | NUMBER | '_' | '-')+;
LOWERCASE_WORD: (LOWERCASE | NUMBER | '_' | '-')+;
UPPERCASE_WORD: (UPPERCASE | NUMBER | '_' | '-')+;
WORD: (LOWERCASE | UPPERCASE | NUMBER | '_' | '-')+;

C_LT: '<';
C_LTE: '<=';
C_GT: '>';
C_GTE: '>=';
C_EQ: '==';

PUNCTUATION: (
		'.'
		| ','
		| '?'
		| '!'
		| ';'
		| ':'
		| '\''
		| '`'
		| '"'
		| '('
		| ')'
		| '&'
		| '%'
		| '$'
		| '@'
		| '*'
		| '-'
		| '_'
		| '<'
		| '>'
	);
