const path = require('path');
const CopyPlugin = require("copy-webpack-plugin");
const TerserPlugin = require('terser-webpack-plugin');
const NodePolyfillPlugin = require("node-polyfill-webpack-plugin")

module.exports = {
  entry: './src/index.ts',
  devtool: "inline-source-map",
  module: {
    rules: [
      {
        test: /\.tsx?$/,
        use: [{
          loader: 'ts-loader',
          options: {
              configFile: "tsconfig.webpack.json"
          }
      }],
        exclude: /node_modules/,
      },
    ],
  },
  output: {
    filename: 'index.js',
    path: path.resolve(__dirname, 'dist/browser'),
    library: {
      name: "UqmFilesParsers",
      type: "window"
    }
  },
  resolve: {
    fallback: { fs: false, assert: require.resolve('assert') },
    extensions: ['.tsx', '.ts', '.js'],
  },
  optimization: {
    minimize: true,
    mangleExports: false,
    minimizer: [
      new TerserPlugin({
        parallel: true,
        terserOptions: {
          keep_classnames: true,
          keep_fnames: true
        },
      }),
    ],
  },
  plugins: [
    new NodePolyfillPlugin()
    /*new CopyPlugin({
      patterns: [],
    }),*/
  ],
};